from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserChangeForm
from django.utils.translation import ugettext_lazy as _


class EmailUserCreationForm(forms.ModelForm):
    """
    A form that creates a user, with no privileges, from the given email and
    password.
    """
    password1 = forms.CharField(label=_('Password'),
                                widget=forms.PasswordInput)
    password2 = forms.CharField(label=_('Password confirmation'),
                                widget=forms.PasswordInput,
                                help_text=_('Enter the same password as '
                                'above, for verification.'))

    class Meta:
        model = get_user_model()
        fields = ('email',)

    def clean_email(self):
        email = self.cleaned_data.get('email')
        try:
            get_user_model().objects.get(email=email)
        except get_user_model().DoesNotExist:
            return email
        raise forms.ValidationError(_('A user with that email already exists.'))

    def clean_password2(self):
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                _('The two password fields did not match.'))
        return password2

    def save(self, commit=True):
        user = super(EmailUserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data['password1'])
        if commit:
            user.save()
        return user


class EmailUserChangeForm(UserChangeForm):
    class Meta:
        model = get_user_model()
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super(EmailUserChangeForm, self).__init__(*args, **kwargs)
        if 'username' in self.fields:
            del self.fields['username']



import binascii
import os
from django.conf import settings
from django.contrib.auth.models import BaseUserManager, AbstractBaseUser
from django.contrib.auth.models import PermissionsMixin
from django.core.mail.message import EmailMultiAlternatives
from django.db import models
from django.template.loader import render_to_string
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.core.mail import send_mail

# Make part of the model eventually, so it can be edited
EXPIRY_PERIOD = 3    # days


def _generate_code():
    return binascii.hexlify(os.urandom(20)).decode('utf-8')


class EmailUserManager(BaseUserManager):
    def _create_user(self, email, password, is_staff, is_superuser,
                     is_verified, **extra_fields):
        """
        Creates and saves a User with a given email and password.
        """
        now = timezone.now()
        if not email:
            raise ValueError('Users must have an email address')
        email = self.normalize_email(email)
        user = self.model(email=email,
                          is_staff=is_staff, is_active=True,
                          is_superuser=is_superuser, is_verified=is_verified,
                          last_login=now, date_joined=now, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        return self._create_user(email, password, False, False, False,
                                 **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        return self._create_user(email, password, True, True, True,
                                 **extra_fields)


class EmailAbstractUser(AbstractBaseUser, PermissionsMixin):
    """
    An abstract base class implementing a fully featured User model with
    admin-compliant permissions.

    Email and password are required. Other fields are optional.
    """
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)
    email = models.EmailField(_('email address'), max_length=255, unique=True)
    is_staff = models.BooleanField(
        _('staff status'), default=False,
        help_text=_('Designates whether the user can log into this '
                    'admin site.'))
    is_active = models.BooleanField(
        _('active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active.  Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    is_verified = models.BooleanField(
        _('verified'), default=False,
        help_text=_('Designates whether this user has completed the email '
                    'verification process to allow login.'))

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['first_name', 'last_name']

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def __str__(self):
        return self.email


class SignupCodeManager(models.Manager):
    def create_signup_code(self, user, ipaddr):
        code = _generate_code()
        signup_code = self.create(user=user, code=code, ipaddr=ipaddr)

        return signup_code

    def set_user_is_verified(self, code):
        try:
            signup_code = SignupCode.objects.get(code=code)
            signup_code.user.is_verified = True
            signup_code.user.save()
            return True
        except SignupCode.DoesNotExist:
            pass

        return False


class PasswordResetCodeManager(models.Manager):
    def create_password_reset_code(self, user):
        code = _generate_code()
        password_reset_code = self.create(user=user, code=code)

        return password_reset_code

    def get_expiry_period(self):
        return EXPIRY_PERIOD


class EmailChangeCodeManager(models.Manager):
    def create_email_change_code(self, user, email):
        code = _generate_code()
        email_change_code = self.create(user=user, code=code, email=email)

        return email_change_code

    def get_expiry_period(self):
        return EXPIRY_PERIOD


def send_multi_format_email(template_prefix, template_ctxt, target_email):
    subject_file = 'authemail/%s_subject.txt' % template_prefix
    txt_file = 'authemail/%s.txt' % template_prefix
    html_file = 'authemail/%s.html' % template_prefix

    subject = render_to_string(subject_file).strip()
    from_email = settings.EMAIL_FROM
    to = target_email
    bcc_email = settings.EMAIL_BCC
    text_content = render_to_string(txt_file, template_ctxt)
    html_content = render_to_string(html_file, template_ctxt)
    msg = EmailMultiAlternatives(subject, text_content, from_email, [to],
                                 bcc=[bcc_email])
    msg.attach_alternative(html_content, 'text/html')
    msg.send()


class AbstractBaseCode(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    code = models.CharField(_('code'), max_length=40, primary_key=True)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True

    def send_email(self, prefix):
        ctxt = {
            'email': self.user.email,
            'first_name': self.user.first_name,
            'last_name': self.user.last_name,
            'code': self.code
        }
        send_multi_format_email(prefix, ctxt, target_email=self.user.email)

    def __str__(self):
        return self.code


class SignupCode(AbstractBaseCode):
    ipaddr = models.GenericIPAddressField(_('ip address'))

    objects = SignupCodeManager()

    def send_signup_email(self):
        prefix = 'signup_email'
        self.send_email(prefix)


class PasswordResetCode(AbstractBaseCode):
    objects = PasswordResetCodeManager()

    def send_password_reset_email(self):
        prefix = 'password_reset_email'
        self.send_email(prefix)


class EmailChangeCode(AbstractBaseCode):
    email = models.EmailField(_('email address'), max_length=255)

    objects = EmailChangeCodeManager()

    def send_email_change_emails(self):
        prefix = 'email_change_notify_previous_email'
        self.send_email(prefix)

        prefix = 'email_change_confirm_new_email'
        ctxt = {
            'email': self.email,
            'code': self.code
        }

        send_multi_format_email(prefix, ctxt, target_email=self.email)



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import ugettext_lazy as _

from authemail.forms import EmailUserCreationForm, EmailUserChangeForm
from authemail.models import SignupCode, PasswordResetCode, EmailChangeCode


class SignupCodeAdmin(admin.ModelAdmin):
    list_display = ('code', 'user', 'ipaddr', 'created_at')
    ordering = ('-created_at',)
    readonly_fields = ('user', 'code', 'ipaddr')

    def has_add_permission(self, request):
        return False


class SignupCodeInline(admin.TabularInline):
    model = SignupCode
    fieldsets = (
        (None, {
            'fields': ('code', 'ipaddr', 'created_at')
        }),
    )
    readonly_fields = ('code', 'ipaddr', 'created_at')

    def has_add_permission(self, request):
        return False


class PasswordResetCodeAdmin(admin.ModelAdmin):
    list_display = ('code', 'user', 'created_at')
    ordering = ('-created_at',)
    readonly_fields = ('user', 'code')

    def has_add_permission(self, request):
        return False


class PasswordResetCodeInline(admin.TabularInline):
    model = PasswordResetCode
    fieldsets = (
        (None, {
            'fields': ('code', 'created_at')
        }),
    )
    readonly_fields = ('code', 'created_at')

    def has_add_permission(self, request):
        return False


class EmailChangeCodeAdmin(admin.ModelAdmin):
    list_display = ('code', 'user', 'email', 'created_at')
    ordering = ('-created_at',)
    readonly_fields = ('user', 'code', 'email')

    def has_add_permission(self, request):
        return False


class EmailChangeCodeInline(admin.TabularInline):
    model = EmailChangeCode
    fieldsets = (
        (None, {
            'fields': ('code', 'email', 'created_at')
        }),
    )
    readonly_fields = ('code', 'email', 'created_at')

    def has_add_permission(self, request):
        return False


class EmailUserAdmin(UserAdmin):
    fieldsets = (
        (None, {'fields': ('email', 'password')}),
        (_('Personal Info'), {'fields': ('first_name', 'last_name')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'password1', 'password2'),
        }),
    )
    form = EmailUserChangeForm
    add_form = EmailUserCreationForm
    inlines = [SignupCodeInline, EmailChangeCodeInline, PasswordResetCodeInline]
    list_display = ('email', 'is_verified', 'first_name', 'last_name',
                    'is_staff')
    search_fields = ('first_name', 'last_name', 'email')
    ordering = ('email',)


admin.site.register(get_user_model(), EmailUserAdmin)
admin.site.register(SignupCode, SignupCodeAdmin)
admin.site.register(PasswordResetCode, PasswordResetCodeAdmin)
admin.site.register(EmailChangeCode, EmailChangeCodeAdmin)



# Generated by Django 2.1.1 on 2020-10-31 20:18

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('authemail', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailChangeCode',
            fields=[
                ('code', models.CharField(max_length=40, primary_key=True, serialize=False, verbose_name='code')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('email', models.EmailField(max_length=255, verbose_name='email address')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 2.1 on 2020-09-27 17:37

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PasswordResetCode',
            fields=[
                ('code', models.CharField(max_length=40, primary_key=True, serialize=False, verbose_name='code')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SignupCode',
            fields=[
                ('code', models.CharField(max_length=40, primary_key=True, serialize=False, verbose_name='code')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('ipaddr', models.GenericIPAddressField(verbose_name='ip address')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.urls import path
from rest_framework.urlpatterns import format_suffix_patterns

from authemail import views


urlpatterns = [
    path('signup/', views.Signup.as_view(), name='authemail-signup'),
    path('signup/verify/', views.SignupVerify.as_view(),
         name='authemail-signup-verify'),

    path('login/', views.Login.as_view(), name='authemail-login'),
    path('logout/', views.Logout.as_view(), name='authemail-logout'),

    path('password/reset/', views.PasswordReset.as_view(),
         name='authemail-password-reset'),
    path('password/reset/verify/', views.PasswordResetVerify.as_view(),
         name='authemail-password-reset-verify'),
    path('password/reset/verified/', views.PasswordResetVerified.as_view(),
         name='authemail-password-reset-verified'),

    path('email/change/', views.EmailChange.as_view(),
         name='authemail-email-change'),
    path('email/change/verify/', views.EmailChangeVerify.as_view(),
         name='authemail-email-change-verify'),

    path('password/change/', views.PasswordChange.as_view(),
         name='authemail-password-change'),

    path('users/me/', views.UserMe.as_view(), name='authemail-me'),
]


urlpatterns = format_suffix_patterns(urlpatterns)



import re
from datetime import timedelta

from django.core import mail
from django.contrib.auth import get_user_model
from django.test.utils import override_settings
from django.urls import reverse
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.test import APITestCase

from authemail.models import SignupCode, PasswordResetCode
from authemail.models import EmailChangeCode


def _get_code_from_email(mail):
    match = re.search(r'\?code=([0-9a-f]+)$', mail.outbox[-1].body, re.MULTILINE)
    if match:
        code = match.group(1)
        return code
    return None


@override_settings(AUTH_EMAIL_VERIFICATION=True)
class SignupTests(APITestCase):
    def setUp(self):
        # A visitor to the site
        self.user_visitor_email = 'visitor@mail.com'
        self.user_visitor_pw = 'visitor'

        # A verified user on the site
        self.user_verified_email = 'user_verified@mail.com'
        self.user_verified_pw = 'user_verified'
        user = get_user_model().objects.create_user(self.user_verified_email, self.user_verified_pw)
        user.is_verified = True
        user.save()

    def test_signup_serializer_errors(self):
        error_dicts = [
            # Email required
            {'payload': {'email': '',
                         'password': self.user_visitor_pw},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'This field may not be blank.')
             },
            # Password required
            {'payload': {'email': self.user_visitor_email,
                         'password': ''},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('password', 'This field may not be blank.')
             },
            # Invalid email
            {'payload': {'email': 'XXX',
                         'password': self.user_visitor_pw},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'Enter a valid email address.')
             },
        ]

        url = reverse('authemail-signup')
        for error_dict in error_dicts:
            response = self.client.post(url, error_dict['payload'])

            self.assertEqual(response.status_code, error_dict['status_code'])
            self.assertEqual(response.data[error_dict['error'][0]][0],
                             error_dict['error'][1])

    def test_signup_email_already_exists(self):
        url = reverse('authemail-signup')
        payload = {
            'email': self.user_verified_email,
            'password': self.user_verified_pw,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'],
                         'Email address already taken.')

    def test_signup_verify_invalid_code(self):
        url = reverse('authemail-signup-verify')
        params = {
            'code': 'XXX',
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

    def test_signup_and_signup_verify(self):
        # Send Signup request
        url = reverse('authemail-signup')
        payload = {
            'email': self.user_visitor_email,
            'password': self.user_visitor_pw,
        }
        response = self.client.post(url, payload)

        # Confirm that new user created, but not verified
        user = get_user_model().objects.latest('id')
        self.assertEqual(user.email, self.user_visitor_email)
        self.assertEqual(user.is_verified, False)

        # Confirm that signup code created
        signup_code = SignupCode.objects.latest('code')
        self.assertEqual(signup_code.user.email, self.user_visitor_email)

        # Confirm that email address in response
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['email'], payload['email'])

        # Confirm that one email sent and that Subject correct
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject,
                         'Verify your email address')

        code = _get_code_from_email(mail)

        # Send Signup Verify request
        url = reverse('authemail-signup-verify')
        params = {
            'code': code,
        }
        response = self.client.get(url, params)

        # Confirm email verified successfully
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Email address verified.')

    def test_signup_without_email_verification(self):

        with self.settings(AUTH_EMAIL_VERIFICATION=False):
            # Send Signup request
            url = reverse('authemail-signup')
            payload = {
                'email': self.user_visitor_email,
                'password': self.user_visitor_pw,
            }
            self.client.post(url, payload)

            # Confirm that new user got created, and was automatically marked as verified
            # (else changing AUTH_EMAIL_VERIFICATION setting later would have horrible consequences)
            user = get_user_model().objects.latest('id')
            self.assertEqual(user.email, self.user_visitor_email)
            self.assertEqual(user.is_verified, True)

            # no verification email sent
            self.assertEqual(len(mail.outbox), 1)
            self.assertEqual(mail.outbox[0].subject,
                             'Welcome')

    def test_signup_twice_then_email_verify(self):
        # Signup mulitple times with same credentials
        num_signups = 2
        self.assertTrue(num_signups > 1)
        for i in range(0, num_signups):
            url = reverse('authemail-signup')
            payload = {
                'email': self.user_visitor_email,
                'password': self.user_visitor_pw,
            }
            response = self.client.post(url, payload)

            self.assertEqual(response.status_code, status.HTTP_201_CREATED)
            self.assertEqual(response.data['email'], payload['email'])
            self.assertEqual(SignupCode.objects.count(), 1)
            self.assertEqual(len(mail.outbox), i+1)
            self.assertEqual(mail.outbox[i].subject,
                             'Verify your email address')

        code = _get_code_from_email(mail)

        # Send Signup Verify request
        url = reverse('authemail-signup-verify')
        params = {
            'code': code,
        }
        response = self.client.get(url, params)

        # Confirm email verified successfully
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Email address verified.')

        # Confirm all signup codes were removed
        self.assertEqual(SignupCode.objects.count(), 0)

        # Confirm multiple signups resulted in only one additional user login
        self.assertEqual(get_user_model().objects.count(), 1+1)


class LoginTests(APITestCase):
    def setUp(self):
        # User who is verified on the site
        self.user_verified_email = 'user_verified@mail.com'
        self.user_verified_pw = 'user_verified'
        self.user_verified = get_user_model().objects.create_user(self.user_verified_email, self.user_verified_pw)
        self.user_verified.is_verified = True
        self.user_verified.save()

        # User who is not verified yet on the site
        self.user_not_verified_email = 'user_not_verified@mail.com'
        self.user_not_verified_pw = 'user_not_verified'
        self.user_not_verified = get_user_model().objects.create_user(self.user_not_verified_email, 'pw')
        self.user_not_verified.save()

        # User who is not active on the site
        self.user_not_active_email = 'user_not_active@mail.com'
        self.user_not_active_pw = 'user_not_active'
        self.user_not_active = get_user_model().objects.create_user(self.user_not_active_email, self.user_not_active_pw)
        self.user_not_active.is_verified = True
        self.user_not_active.is_active = False
        self.user_not_active.save()

    def test_login_serializer_errors(self):
        error_dicts = [
            # Email required
            {'payload': {'email': '',
                         'password': self.user_verified_pw},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'This field may not be blank.')
             },
            # Password required
            {'payload': {'email': self.user_verified_email,
                         'password': ''},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('password', 'This field may not be blank.')
             },
            # Invalid email
            {'payload': {'email': 'XXX',
                         'password': self.user_verified_pw},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'Enter a valid email address.')
             },
        ]

        url = reverse('authemail-login')
        for error_dict in error_dicts:
            response = self.client.post(url, error_dict['payload'])

            self.assertEqual(response.status_code, error_dict['status_code'])
            self.assertEqual(response.data[error_dict['error'][0]][0],
                             error_dict['error'][1])

    def test_login_invalid_credentials(self):
        # Invalid email address
        url = reverse('authemail-login')
        payload = {
            'email': 'XXX@mail.com',
            'password': self.user_verified_pw,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Unable to login with provided credentials.')

        # Invalid password for user
        url = reverse('authemail-login')
        payload = {
            'email': self.user_verified_email,
            'password': 'XXX',
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Unable to login with provided credentials.')

    def test_logout_no_auth_token(self):
        url = reverse('authemail-logout')
        response = self.client.get(url)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Authentication credentials were not provided.')

    def test_logout_invalid_auth_token(self):
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + 'XXX')
        url = reverse('authemail-logout')
        response = self.client.get(url)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'Invalid token.')

    def test_login_logout(self):
        # Log in as the user
        url = reverse('authemail-login')
        payload = {
            'email': self.user_verified_email,
            'password': self.user_verified_pw,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn('token', response.data)

        token = response.data['token']

        # Log out as the user
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        url = reverse('authemail-logout')
        response = self.client.get(url)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'User logged out.')

    def test_login_not_verified_not_active_no_login(self):
        # Not verified user can't login
        url = reverse('authemail-login')
        payload = {
            'email': self.user_not_verified_email,
            'password': self.user_not_verified_pw,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Unable to login with provided credentials.')

        # Not active user can't login
        url = reverse('authemail-login')
        payload = {
            'email': self.user_not_active_email,
            'password': self.user_not_active_pw,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Unable to login with provided credentials.')


class PasswordResetTests(APITestCase):
    def setUp(self):
        # User who is verified on the site
        self.user_verified_email = 'user_verified@mail.com'
        self.user_verified_pw = 'user_verified'
        self.user_verified_pw_reset = 'user_verified reset'
        self.user_verified = get_user_model().objects.create_user(self.user_verified_email, self.user_verified_pw)
        self.user_verified.is_verified = True
        self.user_verified.save()

        # Create auth token for user (so user logged in)
        token = Token.objects.create(user=self.user_verified)
        self.token = token.key

        # User who is not verified yet on the site
        self.user_not_verified_email = 'user_not_verified@mail.com'
        self.user_not_verified_pw = 'user_not_verified'
        self.user_not_verified = get_user_model().objects.create_user(self.user_not_verified_email, 'pw')
        self.user_not_verified.save()

        # User who is verified but not active on the site
        self.user_not_active_email = 'user_not_active@mail.com'
        self.user_not_active_pw = 'user_not_active'
        self.user_not_active = get_user_model().objects.create_user(self.user_not_active_email, self.user_not_active_pw)
        self.user_not_active.is_verified = True
        self.user_not_active.is_active = False
        self.user_not_active.save()

    def test_password_reset_serializer_errors(self):
        error_dicts = [
            # Email required
            {'payload': {'email': ''},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'This field may not be blank.')
             },
            # Invalid email
            {'payload': {'email': 'XXX'},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'Enter a valid email address.')
             },
        ]

        url = reverse('authemail-password-reset')
        for error_dict in error_dicts:
            response = self.client.post(url, error_dict['payload'])

            self.assertEqual(response.status_code, error_dict['status_code'])
            self.assertEqual(response.data[error_dict['error'][0]][0],
                             error_dict['error'][1])

    def test_password_reset_no_user_with_email(self):
        # No user with email address
        url = reverse('authemail-password-reset')
        payload = {
            'email': 'XXX@mail.com'
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Password reset not allowed.')

    def test_password_reset_user_not_verified_not_allowed(self):
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_not_verified_email
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Password reset not allowed.')

    def test_password_reset_user_not_active_not_allowed(self):
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_not_active_email
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Password reset not allowed.')

    def test_password_reset_user_verified_code_created_email_sent(self):
        # Create two past reset codes that aren't used
        password_reset_code_old1 = PasswordResetCode.objects.create_password_reset_code(
            self.user_verified)
        password_reset_code_old2 = PasswordResetCode.objects.create_password_reset_code(
            self.user_verified)
        count = PasswordResetCode.objects.filter(user=self.user_verified).count()
        self.assertEqual(count, 2)

        # Send Password Reset request
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_verified_email,
        }
        response = self.client.post(url, payload)

        # Get password reset code
        password_reset_code = PasswordResetCode.objects.latest('code')

        # Confirm that old password reset codes deleted
        count = PasswordResetCode.objects.filter(user=self.user_verified).count()
        self.assertEqual(count, 1)
        self.assertNotEqual(password_reset_code.code, password_reset_code_old1.code)
        self.assertNotEqual(password_reset_code.code, password_reset_code_old2.code)

        # Confirm that password reset code created
        password_reset_code = PasswordResetCode.objects.latest('code')
        self.assertEqual(password_reset_code.user.email, self.user_verified_email)

        # Confirm that email address in response
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['email'], payload['email'])

        # Confirm that one email sent and that Subject correct
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'Reset Your Password')

    def test_password_reset_verify_invalid_code(self):
        url = reverse('authemail-password-reset-verify')
        params = {
            'code': 'XXX',
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

    def test_password_reset_verify_expired_code(self):
        # Send Password Reset request
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_verified_email,
        }
        self.client.post(url, payload)

        # Get password reset code and make it expire
        password_reset_code = PasswordResetCode.objects.latest('code')
        password_reset_code.created_at += timedelta(days=-(PasswordResetCode.objects.get_expiry_period()+1))
        password_reset_code.save()
        code_lapsed = password_reset_code.code

        # Confirm password reset code_lapsed can't be used
        url = reverse('authemail-password-reset-verify')
        params = {
            'code': code_lapsed,
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

    def test_password_reset_verify_user_verified(self):
        # Send Password Reset request
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_verified_email,
        }
        self.client.post(url, payload)
        password_reset_code = PasswordResetCode.objects.latest('code')
        code = password_reset_code.code

        # Send Password Reset Verify request
        url = reverse('authemail-password-reset-verify')
        params = {
            'code': code,
        }
        response = self.client.get(url, params)

        # Confirm password reset successfully
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Email address verified.')

    def test_password_reset_verified_serializer_errors(self):
        error_dicts = [
            # Password required
            {'payload': {'password': ''},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('password', 'This field may not be blank.')
             },
        ]

        url = reverse('authemail-password-reset-verified')
        for error_dict in error_dicts:
            response = self.client.post(url, error_dict['payload'])

            self.assertEqual(response.status_code, error_dict['status_code'])
            self.assertEqual(response.data[error_dict['error'][0]][0],
                             error_dict['error'][1])

    def test_password_reset_verified_invalid_code(self):
        # Send Password Reset request
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_verified_email,
        }
        self.client.post(url, payload)
        password_reset_code = PasswordResetCode.objects.latest('code')
        code = password_reset_code.code

        # Send Password Reset Verify request
        url = reverse('authemail-password-reset-verify')
        params = {
            'code': code,
        }
        response = self.client.get(url, params)

        # Send Password Reset Verified request
        url = reverse('authemail-password-reset-verified')
        params = {
            'code': 'XXX',
            'password': self.user_verified_pw_reset,
        }
        response = self.client.post(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

    def test_password_reset_verified_user_verified(self):
        # Send Password Reset request for not used code
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_verified_email,
        }
        response = self.client.post(url, payload)
        password_reset_code = PasswordResetCode.objects.latest('code')
        code_not_used = password_reset_code.code

        # Send Password Reset request for used code
        url = reverse('authemail-password-reset')
        payload = {
            'email': self.user_verified_email,
        }
        self.client.post(url, payload)
        password_reset_code = PasswordResetCode.objects.latest('code')
        code_used = password_reset_code.code

        # Send Password Reset Verify request
        url = reverse('authemail-password-reset-verify')
        params = {
            'code': code_used,
        }
        response = self.client.get(url, params)

        # Confirm password reset verify is successful
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Email address verified.')

        # Confirm old codes deleted and one remains
        num_codes = PasswordResetCode.objects.count()
        self.assertEqual(num_codes, 1)

        # Send Password Reset Verified request
        url = reverse('authemail-password-reset-verified')
        payload = {
            'code': code_used,
            'password': self.user_verified_pw_reset,
        }
        response = self.client.post(url, payload)

        # Confirm password reset successfully
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Password reset.')

        # Confirm used code deleted and none remain
        num_codes = PasswordResetCode.objects.count()
        self.assertEqual(num_codes, 0)

        # Confirm password reset code_not_used can't be used again
        url = reverse('authemail-password-reset-verify')
        params = {
            'code': code_not_used,
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

        # Confirm password reset code_used can't be used again
        url = reverse('authemail-password-reset-verify')
        params = {
            'code': code_used,
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

        # Confirm unable to log in with old password
        url = reverse('authemail-login')
        payload = {
            'email': self.user_verified_email,
            'password': self.user_verified_pw,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Unable to login with provided credentials.')

        # Confirm able to log in with new password
        url = reverse('authemail-login')
        payload = {
            'email': self.user_verified_email,
            'password': self.user_verified_pw_reset,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn('token', response.data)


class EmailChangeTests(APITestCase):
    def setUp(self):
        # User who wants to change their email address
        self.user_to_change_email = 'user_to_change@mail.com'
        self.user_to_change = get_user_model().objects.create_user(self.user_to_change_email, 'pw')
        self.user_to_change.is_verified = True
        self.user_to_change.save()

        # Create auth token for user (so user logged in)
        token = Token.objects.create(user=self.user_to_change)
        self.token = token.key

        # User who is verified on the site
        self.user_verified_email = 'user_verified@mail.com'
        self.user_verified_pw = 'user_verified'
        self.user_verified = get_user_model().objects.create_user(self.user_verified_email, 'pw')
        self.user_verified.is_verified = True
        self.user_verified.save()

        # User who is not verified yet on the site
        self.user_not_verified_email = 'user_not_verified@mail.com'
        self.user_not_verified_pw = 'user_not_verified'
        self.user_not_verified = get_user_model().objects.create_user(self.user_not_verified_email, 'pw')
        self.user_not_verified.save()

        # Email address available
        self.available_email = 'available@mail.com'

    def test_email_change_no_auth_token(self):
        url = reverse('authemail-email-change')
        payload = {
            'email': self.user_to_change_email,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Authentication credentials were not provided.')

    def test_email_change_invalid_auth_token(self):
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + 'XXX')
        url = reverse('authemail-email-change')
        response = self.client.post(url)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'Invalid token.')

    def test_email_change_serializer_errors(self):
        error_dicts = [
            # Email required
            {'payload': {'email': ''},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'This field may not be blank.')
             },
            # Email must be valid
            {'payload': {'email': 'abc'},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('email', 'Enter a valid email address.')
             },
        ]

        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        for error_dict in error_dicts:
            response = self.client.post(url, error_dict['payload'])

            self.assertEqual(response.status_code, error_dict['status_code'])
            self.assertEqual(response.data[error_dict['error'][0]][0],
                             error_dict['error'][1])

    def test_email_change_user_verified_so_email_taken(self):
        # Send Email Change request
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        payload = {
            'email': self.user_verified_email,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Email address already taken.')

    def test_email_change_user_not_verified_code_created_and_emails_sent(self):
        # Send Email Change request
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        payload = {
            'email': self.user_not_verified_email,
        }
        response = self.client.post(url, payload)

        # Confirm that email change code created
        email_change_code = EmailChangeCode.objects.latest('code')
        self.assertEqual(email_change_code.user.email, self.user_to_change_email)
        self.assertEqual(email_change_code.email, self.user_not_verified_email)

        # Confirm that email address in response
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['email'], payload['email'])

        # Confirm that two emails sent and that Subject correct
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].subject, 'Notify Previous Email Address')
        self.assertEqual(mail.outbox[1].subject, 'Confirm New Email Address')

    def test_email_change_no_other_user_code_created_and_emails_sent(self):
        # Create two past change codes that aren't used
        email_change_code_old1 = EmailChangeCode.objects.create_email_change_code(
            self.user_to_change, self.user_not_verified_email)
        email_change_code_old2 = EmailChangeCode.objects.create_email_change_code(
            self.user_to_change, self.user_not_verified_email)
        count = EmailChangeCode.objects.filter(user=self.user_to_change).count()
        self.assertEqual(count, 2)

        # Send Email Change request
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        payload = {
            'email': self.available_email,
        }
        response = self.client.post(url, payload)

        # Get email change code
        email_change_code = EmailChangeCode.objects.latest('code')

        # Confirm that old email change codes deleted
        count = EmailChangeCode.objects.filter(user=self.user_to_change).count()
        self.assertEqual(count, 1)
        self.assertNotEqual(email_change_code.code, email_change_code_old1.code)
        self.assertNotEqual(email_change_code.code, email_change_code_old2.code)

        # Confirm that new email change code created properly
        self.assertEqual(email_change_code.user.email, self.user_to_change_email)
        self.assertEqual(email_change_code.email, self.available_email)

        # Confirm that email address in response
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['email'], payload['email'])

        # Confirm that two emails sent and that Subject correct
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].subject, 'Notify Previous Email Address')
        self.assertEqual(mail.outbox[1].subject, 'Confirm New Email Address')

    def test_email_change_verify_invalid_code(self):
        url = reverse('authemail-email-change-verify')
        params = {
            'code': 'XXX',
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

    def test_email_change_verify_expired_code(self):
        # Send Email Change request
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        payload = {
            'email': self.available_email,
        }
        response = self.client.post(url, payload)

        # Get email change code and make it expire
        email_change_code = EmailChangeCode.objects.latest('code')
        email_change_code.created_at += timedelta(days=-(EmailChangeCode.objects.get_expiry_period()+1))
        email_change_code.save()
        code_lapsed = email_change_code.code

        # Confirm email change code_lapsed can't be used
        url = reverse('authemail-email-change-verify')
        params = {
            'code': code_lapsed,
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Unable to verify user.')

    def test_email_change_verify_user_verified_so_email_taken(self):
        # Send Email Change request
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        payload = {
            'email': self.user_not_verified_email,
        }
        response = self.client.post(url, payload)

        # Get email change code and make email that of a verified user
        email_change_code = EmailChangeCode.objects.latest('code')
        email_change_code.email = self.user_verified_email
        email_change_code.save()

        # Confirm email address taken
        url = reverse('authemail-email-change-verify')
        params = {
            'code': email_change_code,
        }
        response = self.client.get(url, params)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['detail'], 'Email address already taken.')

    def test_email_change_verify_user_not_verified_change_email(self):
        # Send Email Change request
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        payload = {
            'email': self.user_not_verified_email,
        }
        response = self.client.post(url, payload)

        # Get email change code and number of users
        email_change_code = EmailChangeCode.objects.latest('code')
        num_users = get_user_model().objects.count()

        # Confirm user_not_verified deleted, email changed, email change code deleted
        url = reverse('authemail-email-change-verify')
        params = {
            'code': email_change_code,
        }
        response = self.client.get(url, params)

        num_users_minus_1 = get_user_model().objects.count()
        self.assertEqual(num_users, num_users_minus_1 + 1)

        try:
            user_exists = get_user_model().objects.get(email=email_change_code.email)
        except get_user_model().DoesNotExist:
            self.fail('User did not get email address changed.')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Email address changed.')

        num_codes = EmailChangeCode.objects.filter(code=email_change_code.code).count()
        self.assertEqual(num_codes, 0)

    def test_email_change_verify_no_other_user_change_email(self):
        # Send Email Change request
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-email-change')
        payload = {
            'email': self.available_email,
        }
        response = self.client.post(url, payload)

        # Get email change code
        email_change_code = EmailChangeCode.objects.latest('code')

        # Confirm email changed email change code deleted
        url = reverse('authemail-email-change-verify')
        params = {
            'code': email_change_code,
        }
        response = self.client.get(url, params)

        try:
            user_exists = get_user_model().objects.get(email=email_change_code.email)
        except get_user_model().DoesNotExist:
            self.fail('User did not get email address changed.')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Email address changed.')

        num_codes = EmailChangeCode.objects.filter(code=email_change_code.code).count()
        self.assertEqual(num_codes, 0)


class PasswordChangeTests(APITestCase):
    def setUp(self):
        # A verified user on the site
        self.user_to_change_email = 'user_to_change@mail.com'
        self.user_to_change_pw = 'pw'
        self.user_to_change = get_user_model().objects.create_user(self.user_to_change_email, self.user_to_change_pw)
        self.user_to_change.is_verified = True
        self.user_to_change.save()

        # Create auth token for user (so user logged in)
        token = Token.objects.create(user=self.user_to_change)
        self.token = token.key

        # New password
        self.user_to_change_pw_new = 'pw new'

    def test_password_change_serializer_errors(self):
        error_dicts = [
            # Password required
            {'payload': {'password': ''},
             'status_code': status.HTTP_400_BAD_REQUEST,
             'error': ('password', 'This field may not be blank.')
             },
        ]

        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-password-change')
        for error_dict in error_dicts:
            response = self.client.post(url, error_dict['payload'])

            self.assertEqual(response.status_code, error_dict['status_code'])
            self.assertEqual(response.data[error_dict['error'][0]][0],
                             error_dict['error'][1])

    def test_password_change_no_auth_token(self):
        url = reverse('authemail-password-change')
        payload = {
            'password': self.user_to_change_pw_new,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Authentication credentials were not provided.')

    def test_password_change_invalid_auth_token(self):
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + 'XXX')
        url = reverse('authemail-password-change')
        response = self.client.post(url)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'Invalid token.')

    def test_password_change(self):
        # Change password
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-password-change')
        payload = {
            'password': self.user_to_change_pw_new,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['success'], 'Password changed.')

        # Confirm unable to log in with old password
        url = reverse('authemail-login')
        payload = {
            'email': self.user_to_change_email,
            'password': self.user_to_change_pw,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Unable to login with provided credentials.')

        # Confirm able to log in with new password
        url = reverse('authemail-login')
        payload = {
            'email': self.user_to_change_email,
            'password': self.user_to_change_pw_new,
        }
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn('token', response.data)


class UserDetailTests(APITestCase):
    def setUp(self):
        # A verified user on the site
        self.user_me_email = 'user_me@mail.com'
        self.user_me = get_user_model().objects.create_user(self.user_me_email, 'pw')
        self.user_me.is_verified = True
        self.user_me.save()

        # Create auth token for user (so user logged in)
        token = Token.objects.create(user=self.user_me)
        self.token = token.key

    def test_user_me_no_auth_token(self):
        url = reverse('authemail-me')
        response = self.client.post(url)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'],
                         'Authentication credentials were not provided.')

    def test_user_me_invalid_auth_token(self):
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + 'XXX')
        url = reverse('authemail-me')
        response = self.client.post(url)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'Invalid token.')

    def test_user_me(self):
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)
        url = reverse('authemail-me')
        response = self.client.get(url)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['email'], self.user_me_email)



from django.contrib.auth import get_user_model
from rest_framework import serializers


class SignupSerializer(serializers.Serializer):
    """
    Don't require email to be unique so visitor can signup multiple times,
    if misplace verification email.  Handle in view.
    """
    email = serializers.EmailField(max_length=255)
    password = serializers.CharField(max_length=128)
    first_name = serializers.CharField(max_length=30, default='',
                                       required=False)
    last_name = serializers.CharField(max_length=30, default='',
                                      required=False)


class LoginSerializer(serializers.Serializer):
    email = serializers.EmailField(max_length=255)
    password = serializers.CharField(max_length=128)


class PasswordResetSerializer(serializers.Serializer):
    email = serializers.EmailField(max_length=255)


class PasswordResetVerifiedSerializer(serializers.Serializer):
    code = serializers.CharField(max_length=40)
    password = serializers.CharField(max_length=128)


class PasswordChangeSerializer(serializers.Serializer):
    password = serializers.CharField(max_length=128)


class EmailChangeSerializer(serializers.Serializer):
    email = serializers.EmailField(max_length=255)


class EmailChangeVerifySerializer(serializers.Serializer):
    email = serializers.EmailField(max_length=255)


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_user_model()
        fields = ('email', 'first_name', 'last_name')



# -*- coding: utf-8 -*-

# See http://pythonhosted.org/an_example_pypi_project/setuptools.html
# See https://packaging.python.org/tutorials/packaging-projects/#uploading-your-project-to-pypi

from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-rest-authemail',
    version='2.0.2',
    author='Celia Oakley',
    author_email='celia.oakley@alumni.stanford.edu',
    description='A RESTful API for user signup and authentication using email addresses',
    keywords=[
        'django', 'python', 'rest', 'rest-framework', 'api',
        'auth', 'authentication', 'email', 'user', 'username',
        'registration', 'signup', 'login', 'logout', 'password',
        'django-rest-framework', 'djangorestframework', 'django-registration',
        'django-email-as-username'
    ],
    url='http://github.com/celiao/django-rest-authemail',
    download_url='https://github.com/celiao/django-rest-authemail/tarball/2.0.2',
    packages=['authemail'],
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        'Django==2.2.13',
        'djangorestframework==3.11.0',
        'requests>=2.3.0',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        'Topic :: Software Development',
        'Topic :: Utilities',
    ],
)



import json
import requests

from django.core.serializers.json import DjangoJSONEncoder


# API class from https://pypi.python.org/pypi/tmdbsimple
class API(object):
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Connection': 'close'
    }
    BASE_PATH = ''
    URLS = {}

    def __init__(self):
        self.base_uri = 'http://127.0.0.1:8000/api'

    def _get_path(self, key):
        return self.BASE_PATH + self.URLS[key]

    def _get_complete_url(self, path):
        return'{base_uri}/{path}'.format(base_uri=self.base_uri, path=path)

    def _request(self, method, path, params=None, payload=None):
        url = self._get_complete_url(path)

        headers = self.headers.copy()
        if 'token' in params:
            headers.update({'Authorization': 'Token ' + params['token']})

        response = requests.request(
            method, url, params=params,
            data=json.dumps(payload, cls=DjangoJSONEncoder) if payload else payload,
            headers=headers)

        response.encoding = 'utf-8'
        return response.json()

    def _GET(self, path, params=None):
        return self._request('GET', path, params=params)

    def _POST(self, path, params=None, payload=None):
        return self._request('POST', path, params=params, payload=payload)

    def _set_attrs_to_values(self, response={}):
        """
        Set attributes to dictionary values so can access via dot notation.
        """
        for key in response.keys():
            setattr(self, key, response[key])


class Authemail(API):
    BASE_PATH = 'accounts'
    URLS = {
        'signup': '/signup/',
        'signup_verify': '/signup/verify/',
        'login': '/login/',
        'logout': '/logout/',
        'password_reset': '/password/reset/',
        'password_reset_verify': '/password/reset/verify/',
        'password_reset_verified': '/password/reset/verified/',
        'email_change': '/email/change/',
        'email_change_verify': '/email/change/verify/',
        'password_change': '/password/change/',
        'users_me': '/users/me/',
    }

    def signup(self, **kwargs):
        path = self._get_path('signup')

        payload = {
            'email': kwargs.pop('email'),
            'password': kwargs.pop('password'),
            'first_name': kwargs.pop('first_name'),
            'last_name': kwargs.pop('last_name'),
        }

        response = self._POST(path, kwargs, payload)

        self._set_attrs_to_values(response)
        return response

    def signup_verify(self, **kwargs):
        path = self._get_path('signup_verify')

        response = self._GET(path, kwargs)

        self._set_attrs_to_values(response)
        return response

    def login(self, **kwargs):
        path = self._get_path('login')

        payload = {
            'email': kwargs.pop('email'),
            'password': kwargs.pop('password'),
        }

        response = self._POST(path, kwargs, payload)

        self._set_attrs_to_values(response)
        return response

    def logout(self, **kwargs):
        path = self._get_path('logout')

        response = self._GET(path, kwargs)

        self._set_attrs_to_values(response)
        return response

    def password_reset(self, **kwargs):
        path = self._get_path('password_reset')

        payload = {
            'email': kwargs.pop('email'),
        }

        response = self._POST(path, kwargs, payload)

        self._set_attrs_to_values(response)
        return response

    def password_reset_verify(self, **kwargs):
        path = self._get_path('password_reset_verify')

        response = self._GET(path, kwargs)

        self._set_attrs_to_values(response)
        return response

    def password_reset_verified(self, **kwargs):
        path = self._get_path('password_reset_verified')

        payload = {
            'code': kwargs.pop('code'),
            'password': kwargs.pop('password'),
        }

        response = self._POST(path, kwargs, payload)

        self._set_attrs_to_values(response)
        return response

    def email_change(self, **kwargs):
        path = self._get_path('email_change')

        payload = {
            'email': kwargs.pop('email'),
        }

        response = self._POST(path, kwargs, payload)

        self._set_attrs_to_values(response)
        return response

    def email_change_verify(self, **kwargs):
        path = self._get_path('email_change_verify')

        response = self._GET(path, kwargs)

        self._set_attrs_to_values(response)
        return response

    def password_change(self, **kwargs):
        path = self._get_path('password_change')

        payload = {
            'password': kwargs.pop('password'),
        }

        response = self._POST(path, kwargs, payload)

        self._set_attrs_to_values(response)
        return response

    def users_me(self, **kwargs):
        path = self._get_path('users_me')

        response = self._GET(path, kwargs)

        self._set_attrs_to_values(response)
        return response



from datetime import date

from django.conf import settings
from django.contrib.auth import authenticate, get_user_model
from django.utils.translation import gettext as _

from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from authemail.models import SignupCode, EmailChangeCode, PasswordResetCode
from authemail.models import send_multi_format_email
from authemail.serializers import SignupSerializer, LoginSerializer
from authemail.serializers import PasswordResetSerializer
from authemail.serializers import PasswordResetVerifiedSerializer
from authemail.serializers import EmailChangeSerializer
from authemail.serializers import PasswordChangeSerializer
from authemail.serializers import UserSerializer


class Signup(APIView):
    permission_classes = (AllowAny,)
    serializer_class = SignupSerializer

    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)

        if serializer.is_valid():
            email = serializer.data['email']
            password = serializer.data['password']
            first_name = serializer.data['first_name']
            last_name = serializer.data['last_name']

            must_validate_email = getattr(settings, "AUTH_EMAIL_VERIFICATION", True)

            try:
                user = get_user_model().objects.get(email=email)
                if user.is_verified:
                    content = {'detail': _('Email address already taken.')}
                    return Response(content, status=status.HTTP_400_BAD_REQUEST)

                try:
                    # Delete old signup codes
                    signup_code = SignupCode.objects.get(user=user)
                    signup_code.delete()
                except SignupCode.DoesNotExist:
                    pass

            except get_user_model().DoesNotExist:
                user = get_user_model().objects.create_user(email=email)

            # Set user fields provided
            user.set_password(password)
            user.first_name = first_name
            user.last_name = last_name
            if not must_validate_email:
                user.is_verified = True
                send_multi_format_email('welcome_email',
                                        {'email': user.email, },
                                        target_email=user.email)
            user.save()

            if must_validate_email:
                # Create and associate signup code
                ipaddr = self.request.META.get('REMOTE_ADDR', '0.0.0.0')
                signup_code = SignupCode.objects.create_signup_code(user, ipaddr)
                signup_code.send_signup_email()

            content = {'email': email, 'first_name': first_name,
                       'last_name': last_name}
            return Response(content, status=status.HTTP_201_CREATED)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class SignupVerify(APIView):
    permission_classes = (AllowAny,)

    def get(self, request, format=None):
        code = request.GET.get('code', '')
        verified = SignupCode.objects.set_user_is_verified(code)

        if verified:
            try:
                signup_code = SignupCode.objects.get(code=code)
                signup_code.delete()
            except SignupCode.DoesNotExist:
                pass
            content = {'success': _('Email address verified.')}
            return Response(content, status=status.HTTP_200_OK)
        else:
            content = {'detail': _('Unable to verify user.')}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)


class Login(APIView):
    permission_classes = (AllowAny,)
    serializer_class = LoginSerializer

    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)

        if serializer.is_valid():
            email = serializer.data['email']
            password = serializer.data['password']
            user = authenticate(email=email, password=password)

            if user:
                if user.is_verified:
                    if user.is_active:
                        token, created = Token.objects.get_or_create(user=user)
                        return Response({'token': token.key},
                                        status=status.HTTP_200_OK)
                    else:
                        content = {'detail': _('User account not active.')}
                        return Response(content,
                                        status=status.HTTP_401_UNAUTHORIZED)
                else:
                    content = {'detail':
                               _('User account not verified.')}
                    return Response(content, status=status.HTTP_401_UNAUTHORIZED)
            else:
                content = {'detail':
                           _('Unable to login with provided credentials.')}
                return Response(content, status=status.HTTP_401_UNAUTHORIZED)

        else:
            return Response(serializer.errors,
                            status=status.HTTP_400_BAD_REQUEST)


class Logout(APIView):
    permission_classes = (IsAuthenticated,)

    def get(self, request, format=None):
        """
        Remove all auth tokens owned by request.user.
        """
        tokens = Token.objects.filter(user=request.user)
        for token in tokens:
            token.delete()
        content = {'success': _('User logged out.')}
        return Response(content, status=status.HTTP_200_OK)


class PasswordReset(APIView):
    permission_classes = (AllowAny,)
    serializer_class = PasswordResetSerializer

    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)

        if serializer.is_valid():
            email = serializer.data['email']

            try:
                user = get_user_model().objects.get(email=email)

                # Delete all unused password reset codes
                PasswordResetCode.objects.filter(user=user).delete()

                if user.is_verified and user.is_active:
                    password_reset_code = \
                        PasswordResetCode.objects.create_password_reset_code(user)
                    password_reset_code.send_password_reset_email()
                    content = {'email': email}
                    return Response(content, status=status.HTTP_201_CREATED)

            except get_user_model().DoesNotExist:
                pass

            # Since this is AllowAny, don't give away error.
            content = {'detail': _('Password reset not allowed.')}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)

        else:
            return Response(serializer.errors,
                            status=status.HTTP_400_BAD_REQUEST)


class PasswordResetVerify(APIView):
    permission_classes = (AllowAny,)

    def get(self, request, format=None):
        code = request.GET.get('code', '')

        try:
            password_reset_code = PasswordResetCode.objects.get(code=code)

            # Delete password reset code if older than expiry period
            delta = date.today() - password_reset_code.created_at.date()
            if delta.days > PasswordResetCode.objects.get_expiry_period():
                password_reset_code.delete()
                raise PasswordResetCode.DoesNotExist()

            content = {'success': _('Email address verified.')}
            return Response(content, status=status.HTTP_200_OK)
        except PasswordResetCode.DoesNotExist:
            content = {'detail': _('Unable to verify user.')}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)


class PasswordResetVerified(APIView):
    permission_classes = (AllowAny,)
    serializer_class = PasswordResetVerifiedSerializer

    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)

        if serializer.is_valid():
            code = serializer.data['code']
            password = serializer.data['password']

            try:
                password_reset_code = PasswordResetCode.objects.get(code=code)
                password_reset_code.user.set_password(password)
                password_reset_code.user.save()

                # Delete password reset code just used
                password_reset_code.delete()

                content = {'success': _('Password reset.')}
                return Response(content, status=status.HTTP_200_OK)
            except PasswordResetCode.DoesNotExist:
                content = {'detail': _('Unable to verify user.')}
                return Response(content, status=status.HTTP_400_BAD_REQUEST)

        else:
            return Response(serializer.errors,
                            status=status.HTTP_400_BAD_REQUEST)


class EmailChange(APIView):
    permission_classes = (IsAuthenticated,)
    serializer_class = EmailChangeSerializer

    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)

        if serializer.is_valid():
            user = request.user

            # Delete all unused email change codes
            EmailChangeCode.objects.filter(user=user).delete()

            email_new = serializer.data['email']

            try:
                user_with_email = get_user_model().objects.get(email=email_new)
                if user_with_email.is_verified:
                    content = {'detail': _('Email address already taken.')}
                    return Response(content, status=status.HTTP_400_BAD_REQUEST)
                else:
                    # If the account with this email address is not verified,
                    # give this user a chance to verify and grab this email address
                    raise get_user_model().DoesNotExist

            except get_user_model().DoesNotExist:
                email_change_code = EmailChangeCode.objects.create_email_change_code(user, email_new)

                email_change_code.send_email_change_emails()

                content = {'email': email_new}
                return Response(content, status=status.HTTP_201_CREATED)

        else:
            return Response(serializer.errors,
                            status=status.HTTP_400_BAD_REQUEST)


class EmailChangeVerify(APIView):
    permission_classes = (AllowAny,)

    def get(self, request, format=None):
        code = request.GET.get('code', '')

        try:
            # Check if the code exists.
            email_change_code = EmailChangeCode.objects.get(code=code)

            # Check if the code has expired.
            delta = date.today() - email_change_code.created_at.date()
            if delta.days > EmailChangeCode.objects.get_expiry_period():
                email_change_code.delete()
                raise EmailChangeCode.DoesNotExist()

            # Check if the email address is being used by a verified user.
            try:
                user_with_email = get_user_model().objects.get(email=email_change_code.email)
                if user_with_email.is_verified:
                    # Delete email change code since won't be used
                    email_change_code.delete()

                    content = {'detail': _('Email address already taken.')}
                    return Response(content, status=status.HTTP_400_BAD_REQUEST)
                else:
                    # If the account with this email address is not verified,
                    # delete the account (and signup code) because the email
                    # address will be used for the user who just verified.
                    user_with_email.delete()
            except get_user_model().DoesNotExist:
                pass

            # If all is well, change the email address.
            email_change_code.user.email = email_change_code.email
            email_change_code.user.save()

            # Delete email change code just used
            email_change_code.delete()

            content = {'success': _('Email address changed.')}
            return Response(content, status=status.HTTP_200_OK)
        except EmailChangeCode.DoesNotExist:
            content = {'detail': _('Unable to verify user.')}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)


class PasswordChange(APIView):
    permission_classes = (IsAuthenticated,)
    serializer_class = PasswordChangeSerializer

    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)

        if serializer.is_valid():
            user = request.user

            password = serializer.data['password']
            user.set_password(password)
            user.save()

            content = {'success': _('Password changed.')}
            return Response(content, status=status.HTTP_200_OK)

        else:
            return Response(serializer.errors,
                            status=status.HTTP_400_BAD_REQUEST)


class UserMe(APIView):
    permission_classes = (IsAuthenticated,)
    serializer_class = UserSerializer

    def get(self, request, format=None):
        return Response(self.serializer_class(request.user).data)
